import * as assert from 'assert';
import * as httpMocks from 'node-mocks-http';
import { addAuction, listAuctions, bidInAuction } from './routes';

describe('routes', function() {

  it('end-to-end', function() {
    const req1 = httpMocks.createRequest({method: 'GET', url: '/api/list'});
    const res1 = httpMocks.createResponse();
    listAuctions(req1, res1);

    assert.strictEqual(res1._getStatusCode(), 200);
    assert.deepStrictEqual(res1._getData(), {auctions: []});

    const req2 = httpMocks.createRequest({method: 'POST', url: '/api/add',
        query: {
          name: "cabinet",
          description: "an oak cabinet",
          seller: "Fred",
          minBid: "2",
          minutes: "10"
        }});
    const res2 = httpMocks.createResponse();
    addAuction(req2, res2);

    assert.strictEqual(res2._getStatusCode(), 200);
    const data2 = res2._getData();
    assert.deepStrictEqual(data2.name, "cabinet");
    assert.deepStrictEqual(data2.description, "an oak cabinet");
    assert.deepStrictEqual(data2.seller, "Fred");
    assert.deepStrictEqual(data2.maxBid, 2);

    const minRemain = (data2.endTime - Date.now()) / 1000 / 60;
    assert.ok(Math.abs(minRemain - 10) < 1e-3);

    const req3 = httpMocks.createRequest({method: 'GET', url: '/api/list'});
    const res3 = httpMocks.createResponse();
    listAuctions(req3, res3);

    assert.strictEqual(res3._getStatusCode(), 200);
    assert.deepStrictEqual(res3._getData(), {auctions: [data2]});

    const req4 = httpMocks.createRequest({method: 'POST', url: '/api/bid',
        query: {name: "cabinet", bidder: "Bob", amount: "5"}});
    const res4 = httpMocks.createResponse();
    bidInAuction(req4, res4);

    data2.maxBid = 5;
    data2.maxBidder = "Bob";
    assert.strictEqual(res4._getStatusCode(), 200);
    assert.deepStrictEqual(res4._getData(), data2);

    const req5 = httpMocks.createRequest({method: 'POST', url: '/api/add',
        query: {
          name: "bicycle",
          description: "a red bicycle",
          seller: "Bob",
          minBid: "100",
          minutes: "15"
        }});
    const res5 = httpMocks.createResponse();
    addAuction(req5, res5);

    assert.strictEqual(res5._getStatusCode(), 200);
    const data5 = res5._getData();
    assert.deepStrictEqual(data5.name, "bicycle");
    assert.deepStrictEqual(data5.description, "a red bicycle");
    assert.deepStrictEqual(data5.seller, "Bob");
    assert.deepStrictEqual(data5.maxBid, 100);

    const minRemain5 = (data5.endTime - Date.now()) / 1000 / 60;
    assert.ok(Math.abs(minRemain5 - 15) < 1e-3);

    const req6 = httpMocks.createRequest({method: 'GET', url: '/api/list'});
    const res6 = httpMocks.createResponse();
    listAuctions(req6, res6);

    assert.strictEqual(res6._getStatusCode(), 200);
    assert.deepStrictEqual(res6._getData(), {auctions: [data2, data5]});
  });

});
